package com.thomsonreuters.dss.api.example;
/*
 * ====================================================================
 * This Java example demonstrate how to make a HTTPS request to DSS2
 * to create/delete Report Templates, get Report Templates.
 * 
 *  To run this example, you will need to provide a DSS2 username and 
 *  password.
 * ====================================================================
 *
 */



import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONOrderedObject;

public class DSS2LegalEntityDetailReportTemplatesClient {
	private String sessionToken ="";
	private String urlHost = "https://hosted.datascopeapi.reuters.com/RestApi/v1";
	
	private CloseableHttpClient httpclient = HttpClientBuilder.create().build();
	
    public static void main(String[] args) throws Exception {

    	if(args.length < 2) {
    		System.out.println("Please enter DSS2 Username as 1st command line parameter, Password as 2nd");
    		System.exit(-1);
    	}
    	
    	DSS2LegalEntityDetailReportTemplatesClient dss2 = new DSS2LegalEntityDetailReportTemplatesClient();
    	dss2.getSessionToken(args[0], args[1]);
    	promptEnterKey();
    	
    	// Create a Report Template
    	String reportTemplateId = dss2.createReportTemplate("LegalEntity_TEMPLATE");
    	promptEnterKey();
    	
    	// Get Report Template by providing an Id
    	dss2.getReportTemplate( reportTemplateId);
    	promptEnterKey();
    	
    	// Delete Report Template by providing an Id    	
    	dss2.deleteReportTemplate( reportTemplateId );
    	promptEnterKey();
    	
    	dss2.getAllReportTemplates();
    }
    
    /**
     * Request DSS2 for a Session Token (24 hour life)
     * 
     * @param username
     * @param psswd
     */
    public void getSessionToken(String username, String psswd) {
    	
    	try {	
	        HttpPost httppost = new HttpPost(urlHost + "/Authentication/RequestToken"); 
	        
	        // Add the Credentials
	        // {"Credentials":{"Password":"{your password here}","Username": {your user name here} }}
	        JSONObject credentials = new JSONObject();
	        JSONObject jsonObject = new JSONObject();	        
			jsonObject.put("Password",  psswd);	
	        jsonObject.put("Username",  username);
	        credentials.put("Credentials", jsonObject);
	        
	        httppost.addHeader("content-type", "application/json; charset=UTF-8");      
	        
	        StringEntity params =new StringEntity(credentials.toString());
	        
	        httppost.setEntity(params);
	
	        ResponseHandler<String> responseHandler=new BasicResponseHandler();
	
	        String strResponse = httpclient.execute(httppost, responseHandler);
	        JSONObject jsonResponse=new JSONObject(strResponse);
	        
	        sessionToken = jsonResponse.get("value").toString();
	        System.out.println("Session Token (expires in 24 hours): "+ sessionToken);

		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    

    /**
     * Create a Report Template
     */
    public String createReportTemplate(String reportTemplateName) {
    	
    	String reportTemplateId = "";
    	
    	try {	
    		
	        HttpPost httppost = new HttpPost(urlHost + "/Extractions/LegalEntityDetailReportTemplates"); 
	        
	        httppost.addHeader("content-type", "application/json;odata.metadata=minimal");  
	        httppost.addHeader("Authorization", "Token "+sessionToken);
	        
	        JSONOrderedObject legalTemplateJSONObject = new JSONOrderedObject();
	        legalTemplateJSONObject.put("@odata.type", "#ThomsonReuters.Dss.Api.Extractions.ReportTemplates.LegalEntityDetailReportTemplate");
	        legalTemplateJSONObject.put("ShowColumnHeaders", false);
	        legalTemplateJSONObject.put("Name", reportTemplateName);
	        JSONArray headersArray = new JSONArray();
	        legalTemplateJSONObject.put("Headers", headersArray);
	        JSONArray trailersArray = new JSONArray();
	        legalTemplateJSONObject.put("Trailers", trailersArray);
	        
	        // ContentFields: Add fields
	        JSONArray contentFieldNamesArray = new JSONArray();	
	        
	        JSONObject cfJSONObject = new JSONObject();
	        cfJSONObject.put("FieldName", "CIK");	       
	        cfJSONObject.put( "Format", JSONObject.NULL);	        	     
	        contentFieldNamesArray.put(cfJSONObject);
	        
	        	        
	        JSONObject cfJSONObject2 = new JSONObject();
	        cfJSONObject2.put("FieldName", "Entity URL");	       
	        cfJSONObject2.put( "Format", JSONObject.NULL);
	        contentFieldNamesArray.put(cfJSONObject2);
	        
	        JSONObject cfJSONObject3 = new JSONObject();
	        cfJSONObject3.put("FieldName", "ID");	       
	        cfJSONObject3.put( "Format", JSONObject.NULL);
	        contentFieldNamesArray.put(cfJSONObject3);
	        
	        JSONObject cfJSONObject4 = new JSONObject();
	        cfJSONObject4.put("FieldName", "Company/Registration Number");	       
	        cfJSONObject4.put( "Format", JSONObject.NULL);
	        contentFieldNamesArray.put(cfJSONObject4);
	        
	        JSONObject cfJSONObject5 = new JSONObject();
	        cfJSONObject5.put("FieldName", "Date of Incorporation");	       
	        cfJSONObject5.put( "Format", JSONObject.NULL);
	        contentFieldNamesArray.put(cfJSONObject5);	        
	        
	        legalTemplateJSONObject.put("ContentFields", contentFieldNamesArray);
	        	        
	        StringEntity params =new StringEntity(legalTemplateJSONObject.toString());
	        
	        httppost.setEntity(params);
	
	        ResponseHandler<String> responseHandler=new BasicResponseHandler();
	
	        String strResponse = httpclient.execute(httppost, responseHandler);
	        JSONObject jsonResponse=new JSONObject(strResponse);
	        
	        StringWriter out = new StringWriter();
	        jsonResponse.write(out);	
	        System.out.println("Create Report Template: JSON response: "+ out);
	        reportTemplateId = jsonResponse.getString("ReportTemplateId");
	        
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	
    	return reportTemplateId;
    }
    
    
    /**
     * Delete report template with this reportTemplateId
     * @param reportTemplateId
     * 
     */
    public void deleteReportTemplate(String reportTemplateId) {
    	try {
    		
	        String urlDel = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";
			
			HttpDelete request = new HttpDelete(urlDel);
			request.addHeader("Authorization", "Token "+sessionToken);
			
			System.out.println("Delete Report Template with this id: "+ reportTemplateId);
			HttpResponse response = httpclient.execute(request);
				
			System.out.println("Response Code : " 
					+ response.getStatusLine().getStatusCode());
			
			// it'd receive response code 204 with no content	
			if (response.getStatusLine().getStatusCode() == 204){
				System.out.println("Delete ReportTemplate ID " + reportTemplateId + " completed.");
			}

			
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
    }
    
    
    /**
     * Get Report Template for this reportTemplateId
     * @param reportTemplateId
     * 
     */
    public void getReportTemplate(String reportTemplateId) {
    	try {
	        String urlGet = urlHost + "/Extractions/ReportTemplates('"+reportTemplateId+"')";
			HttpGet request = new HttpGet(urlGet);
			request.addHeader("Authorization", "Token "+sessionToken);
			
			System.out.println("Retrieve Report Template with this Id: "+ reportTemplateId);
			HttpResponse response = httpclient.execute(request);
				
			// it'd receive response code 404, if no template found for that id.
			System.out.println("Response Code : " 
					+ response.getStatusLine().getStatusCode());
	
			BufferedReader rd = new BufferedReader(
					new InputStreamReader(response.getEntity().getContent()));
	
			StringBuffer result = new StringBuffer();
			String line = "";
			while ((line = rd.readLine()) != null) {
				result.append(line);
			}
			
			System.out.println("ReportTemplate received: "+result);			
			
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
    }
    
    
    /**
     * Get all Report Templates
     * 
     */
    public void getAllReportTemplates() {
    	try {
	        String urlGet = urlHost + "/Extractions/ReportTemplates";
			HttpGet request = new HttpGet(urlGet);
			request.addHeader("Authorization", "Token "+sessionToken);
			
			HttpResponse response = httpclient.execute(request);
				
			System.out.println("Response Code : " 
					+ response.getStatusLine().getStatusCode());
	
			BufferedReader rd = new BufferedReader(
					new InputStreamReader(response.getEntity().getContent()));
	
			StringBuffer result = new StringBuffer();
			String line = "";
			while ((line = rd.readLine()) != null) {
				result.append(line);
			}
			
			System.out.println("ReportTemplates received: "+result);
			
			JSONObject jsonGetResponse=new JSONObject(result.toString());
			
			String userDetail = jsonGetResponse.get("value").toString(); 
			System.out.println("All Report Templates: " +userDetail);
			
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
    }
    
    public static void promptEnterKey(){
		System.out.println("Press \"ENTER\" to continue...");
		Scanner scanner = new Scanner(System.in);
		scanner.nextLine();
    }    
} //EOF
